REM ---------------------------------------------------------------------------
REM  Trivadis - http://www.trivadis.com
REM ---------------------------------------------------------------------------
REM  Group/Privileges.: DBA
REM  Script-Name......: ex_execution_plan_stability_11g.sql
REM  Developer........: Christian Antognini (christian.antognini@trivadis.com)
REM  Date.............: November 2007
REM  Version..........: Oracle Database 11g Release 1
REM  Description......: This script, along with the other one with the suffix
REM                     "10g" is used to demonstrate how to create a SQL
REM                     tuning set in 10gR2, move it to 11gR1 and create an
REM                     execution plan baseline based on it.
REM  Usage............: 1) 10gR2: run ex_execution_plan_stability_10g.sql
REM                     2) 10gR2: export the staging table 
REM                     3) 11gR1: import the staging table
REM                     4) 11gR1: run ex_execution_plan_stability_11g.sql
REM                     
REM  Input parameters.: 
REM  Output...........: 
REM  Called by........:
REM  Requirements.....: 
REM  Remarks..........: In 11gR2 the structure of the staging table is different.
REM ---------------------------------------------------------------------------
REM Changes:
REM DD.MM.YYYY Developer Change
REM ---------------------------------------------------------------------------
REM 12.12.2011 ChA       Changed header + added remark about 11gR2
REM ---------------------------------------------------------------------------

SET ECHO ON TERMOUT ON SERVEROUTPUT OFF FEEDBACK OFF LONG 100000 PAGESIZE 100 LINESIZE 100

COLUMN PLAN_TABLE_OUTPUT FORMAT A80
COLUMN SQL_TEXT FORMAT A45
COLUMN SQL_HANDLE FORMAT A25

DROP TABLE t;

execute dbms_sqltune.drop_sqlset('10GR2_SET')

DROP TABLE my_stgtab PURGE;

execute dbms_sqltune.drop_sqlset('10GR2_SET')

CLEAR SCREEN

REM
REM create test table and gather statistics
REM

CREATE TABLE t (id, n, pad, CONSTRAINT t_pk PRIMARY KEY (id))
AS 
SELECT rownum, rownum, rpad('*',500,'*')
FROM dba_objects WHERE rownum <= 1000;

BEGIN
  dbms_stats.gather_table_stats(
    ownname => user, 
    tabname => 't', 
    estimate_percent => 100, 
    method_opt => 'for all columns size 254',
    cascade => TRUE
  );
END;
/

PAUSE

REM
REM import staging table with Data Pump
REM

REM impdp full=y

PAUSE

REM
REM import SQL tuning set
REM

BEGIN
  dbms_sqltune.unpack_stgtab_sqlset(
    sqlset_name          => '10GR2_SET',
    sqlset_owner         => user,
    replace              => FALSE,
    staging_table_name   => 'MY_STGTAB',
    staging_schema_owner => user
  );
END;
/

PAUSE

REM
REM display content of the SQL tuning set
REM

SELECT * 
FROM table(dbms_xplan.display_sqlset('10GR2_SET', '2n8tjyr1nthf5'));

PAUSE

REM
REM load baseline from SQL tuning set
REM

DECLARE
  dummy PLS_INTEGER;
BEGIN
  dummy := dbms_spm.load_plans_from_sqlset(sqlset_name => '10GR2_SET');
END;
/

REM
REM the new execution plan is added to the baseline and accepted
REM

SELECT sql_handle, sql_text, enabled, accepted FROM dba_sql_plan_baselines;

PAUSE

REM
REM display the execution plans of the baseline
REM

SELECT * FROM table(dbms_xplan.display_sql_plan_baseline('&sql_handle', NULL, 'basic'));

PAUSE

REM
REM create an index to avoid the full table scan
REM

CREATE INDEX i ON t (n);

PAUSE

REM
REM the index is not used because the baseline is used
REM

EXPLAIN PLAN FOR SELECT count(pad) FROM t WHERE n = 42;

SELECT * FROM table(dbms_xplan.display(NULL, NULL, 'basic note'));

PAUSE

REM
REM execute twice the query... then the query optimizer notice that
REM another execution plan could be used to process the query
REM

SELECT count(pad) FROM t WHERE n = 42;

SELECT count(pad) FROM t WHERE n = 42;

PAUSE

REM
REM the new execution plan is added to the baseline but not accepted
REM

SELECT sql_handle, sql_text, enabled, accepted FROM dba_sql_plan_baselines;

PAUSE

REM
REM display the execution plans of the baseline
REM

SELECT * FROM table(dbms_xplan.display_sql_plan_baseline('&sql_handle', NULL, 'basic'));

PAUSE

REM
REM evolve the baseline 
REM

SELECT dbms_spm.evolve_sql_plan_baseline('&sql_handle') FROM dual;

PAUSE

SELECT sql_handle, sql_text, enabled, accepted FROM dba_sql_plan_baselines;

PAUSE

REM
REM check if the new execution plan is used
REM

EXPLAIN PLAN FOR SELECT count(pad) FROM t WHERE n = 42;

SELECT * FROM table(dbms_xplan.display(NULL, NULL, 'basic note'));

PAUSE

REM
REM cleanup
REM

REM remove all baselines created in the last 15 minutes

DECLARE
  ret PLS_INTEGER;
BEGIN
  FOR c IN (SELECT DISTINCT sql_handle 
            FROM dba_sql_plan_baselines 
            WHERE creator = user
            AND created > systimestamp - to_dsinterval('0 00:15:00'))
  LOOP
    ret := dbms_spm.drop_sql_plan_baseline(c.sql_handle);
  END LOOP;
END;
/

REM remove SQL tuning set

execute dbms_sqltune.drop_sqlset('10GR2_SET')

REM remove stating table

DROP TABLE my_stgtab PURGE;
