REM ---------------------------------------------------------------------------
REM  Trivadis - http://www.trivadis.com
REM ---------------------------------------------------------------------------
rem  Group/Privileges.: DBA
rem  Script-Name......: ex_incremental_stats.sql
rem  Developer........: MaM (maurice.mueller@trivadis.com)
rem  Date.............: October 2007
rem  Version..........: Oracle Database 11g Release 1 and Release 2
rem  Description......: The aim of the script is twofold. First, to show how to
rem                     activate incremental statistics for a partitioned table.
rem                     Second, to show how incremental statistics are gathered.
rem  Usage............: 
rem                     
rem  Input parameters.: 
rem  Output...........: 
rem  Called by........:
rem  Requirements.....:
rem  Remarks..........: 
rem ---------------------------------------------------------------------------
rem Changes:
rem DD.MM.YYYY Developer Change
rem ---------------------------------------------------------------------------
rem 13.11.2007 ChA       Formatting + added insert statement to show difference
rem                      in cardinality as well + reduced number of rows
REM 12.12.2011 ChA       Changed header + added cleanup + handling 11.2.0.2
REM                      structure of the sys.wri$_optstat_synopsis$ table
rem ---------------------------------------------------------------------------

SET ECHO ON TERMOUT ON SERVEROUTPUT ON PAGESIZE 100 LINESIZE 80

column partition_name format a15
column column_name format a15
column subobject_name format a15
column statistic_name format a15

DROP TABLE t PURGE;

CLEAR SCREEN

REM
REM create test table, insert data and gather statistics
REM

CREATE TABLE t (id NUMBER, dat DATE, nb number)
PARTITION BY RANGE (dat)
(
  PARTITION t_2007_01 VALUES LESS THAN (to_date('01.02.2007','dd.mm.yyyy')),
  PARTITION t_2007_02 VALUES LESS THAN (to_date('01.03.2007','dd.mm.yyyy')),
  PARTITION t_2007_03 VALUES LESS THAN (to_date('01.04.2007','dd.mm.yyyy'))
);

INSERT INTO t
SELECT rownum, add_months(to_date('01.01.2007', 'dd.mm.yyyy') , mod(rownum, 3)), mod(rownum, 10) 
FROM all_objects
WHERE rownum <= 9000;

COMMIT;

BEGIN
  dbms_stats.gather_table_stats(
    ownname          => user, 
    tabname          => 't' , 
    method_opt       => 'for all columns size 1', 
    estimate_percent => 100
  );
END;
/

PAUSE

REM
REM Shows the number of distinct values stored in the table wri$_optstat_synopsis
REM
REM In this case because incremental is not enabled no row will be returned
REM

describe sys.wri$_optstat_synopsis$

PAUSE

REM works up to 11.2.0.1

SELECT count(*)
FROM sys.wri$_optstat_synopsis$
WHERE synopsis# IN (SELECT sh.synopsis# 
                    FROM sys.wri$_optstat_synopsis_head$ sh
                    WHERE bo# IN (SELECT object_id
                                  FROM sys.user_objects
                                  WHERE object_name = 'T')
                    AND intcol# IN (SELECT column_id
                                    FROM sys.user_tab_columns
                                    WHERE TABLE_NAME = 'T' 
                                    AND column_name = 'NB')
                   )
GROUP BY synopsis#
ORDER BY synopsis#;

REM works as of 11.2.0.2

SELECT count(*)
FROM sys.wri$_optstat_synopsis$
WHERE bo# IN (SELECT object_id
              FROM sys.user_objects
              WHERE object_name = 'T')
AND intcol# IN (SELECT column_id
                FROM sys.user_tab_columns
                WHERE TABLE_NAME = 'T'
                AND column_name = 'NB')
GROUP BY group#
ORDER BY group#;

PAUSE

REM
REM enable incremental and gather statistics
REM

BEGIN
  dbms_stats.set_table_prefs(
    ownname => user, 
    tabname => 't', 
    pname   => 'incremental',
    pvalue  => 'true'
  );
  dbms_stats.gather_table_stats(
    ownname          => user, 
    tabname          => 't', 
    method_opt       => 'for all columns size 1', 
    estimate_percent => 100
  );
END;
/

PAUSE

REM
REM shows the number of distinct values stored in the table wri$_optstat_synopsis
REM

REM works up to 11.2.0.1

SELECT count(*)
FROM sys.wri$_optstat_synopsis$
WHERE synopsis# IN (SELECT sh.synopsis# 
                    FROM sys.wri$_optstat_synopsis_head$ sh
                    WHERE bo# IN (SELECT object_id
                                  FROM sys.user_objects
                                  WHERE object_name = 'T')
                    AND intcol# IN (SELECT column_id
                                    FROM sys.user_tab_columns
                                    WHERE TABLE_NAME = 'T' 
                                    AND column_name = 'NB')
                   )
GROUP BY synopsis#
ORDER BY synopsis#;

REM works as of 11.2.0.2

SELECT count(*)
FROM sys.wri$_optstat_synopsis$
WHERE bo# IN (SELECT object_id
              FROM sys.user_objects
              WHERE object_name = 'T')
AND intcol# IN (SELECT column_id
                FROM sys.user_tab_columns
                WHERE TABLE_NAME = 'T'
                AND column_name = 'NB')
GROUP BY group#
ORDER BY group#;

PAUSE

REM
REM show statistics in data dictionary
REM

SELECT num_rows, sample_size, to_char(last_analyzed,'hh24:mi:ss') AS time 
FROM user_tables 
WHERE table_name = 'T';

SELECT partition_name, num_rows, sample_size, to_char(last_analyzed,'hh24:mi:ss') AS time 
FROM user_tab_partitions 
WHERE table_name = 'T'
ORDER BY partition_name;

PAUSE

SELECT column_name, num_distinct 
FROM user_tab_columns 
WHERE table_name = 'T';

SELECT  partition_name, column_name, num_distinct 
FROM user_part_col_statistics 
WHERE table_name = 'T' 
AND column_name = 'NB' 
ORDER BY partition_name;

PAUSE

REM
REM update a single partition, gather statistics and show that
REM only the modified partition is accessed
REM

UPDATE t PARTITION (t_2007_01) SET nb = 11 WHERE mod(id,7)= 0; 
UPDATE t PARTITION (t_2007_01) SET nb = 12 WHERE mod(id,7)= 1; 
INSERT INTO t SELECT * FROM t PARTITION (t_2007_01) WHERE rownum <= 33;
COMMIT;

PAUSE

SELECT subobject_name, statistic_name, value 
FROM v$segment_statistics 
WHERE object_name = 'T' 
AND owner = user 
AND statistic_name = 'logical reads';

BEGIN
  dbms_stats.gather_schema_stats(
    ownname          => user, 
    options          => 'gather stale', 
    method_opt       => 'for all columns size 1', 
    estimate_percent => 100
  );
END;
/

SELECT subobject_name, statistic_name, value 
FROM v$segment_statistics 
WHERE object_name = 'T' 
AND owner = user 
AND statistic_name = 'logical reads';

PAUSE

REM
REM shows the number of distinct values stored in the table wri$_optstat_synopsis
REM

REM works up to 11.2.0.1

SELECT count(*)
FROM sys.wri$_optstat_synopsis$
WHERE synopsis# IN (SELECT sh.synopsis# 
                    FROM sys.wri$_optstat_synopsis_head$ sh
                    WHERE bo# IN (SELECT object_id
                                  FROM sys.user_objects
                                  WHERE object_name = 'T')
                    AND intcol# IN (SELECT column_id
                                    FROM sys.user_tab_columns
                                    WHERE TABLE_NAME = 'T' 
                                    AND column_name = 'NB')
                   )
GROUP BY synopsis#
ORDER BY synopsis#;

REM works as of 11.2.0.2

SELECT count(*)
FROM sys.wri$_optstat_synopsis$
WHERE bo# IN (SELECT object_id
              FROM sys.user_objects
              WHERE object_name = 'T')
AND intcol# IN (SELECT column_id
                FROM sys.user_tab_columns
                WHERE TABLE_NAME = 'T'
                AND column_name = 'NB')
GROUP BY group#
ORDER BY group#;

PAUSE

REM
REM show statistics in data dictionary, only partition statistics changed
REM

SELECT num_rows, sample_size, to_char(last_analyzed,'hh24:mi:ss') AS time 
FROM user_tables 
WHERE table_name = 'T';

SELECT partition_name, num_rows, sample_size, to_char(last_analyzed,'hh24:mi:ss') AS time 
FROM user_tab_partitions 
WHERE table_name = 'T'
ORDER BY partition_name;

PAUSE

SELECT column_name, num_distinct 
FROM user_tab_columns 
WHERE table_name = 'T';

SELECT  partition_name, column_name, num_distinct 
FROM user_part_col_statistics 
WHERE table_name = 'T' 
AND column_name = 'NB' 
ORDER BY partition_name;

PAUSE

REM
REM update a single partition, gather statistics and show that
REM only the modified partition is accessed
REM

UPDATE t PARTITION (t_2007_01) SET nb = 9 WHERE nb = 11; 
COMMIT;

PAUSE

SELECT subobject_name, statistic_name, value 
FROM v$segment_statistics 
WHERE object_name = 'T' 
AND owner = user 
AND statistic_name = 'logical reads';

BEGIN
  dbms_stats.gather_schema_stats(
    ownname          => user, 
    options          => 'gather stale', 
    method_opt       => 'for all columns size 1', 
    estimate_percent => 100
  );
END;
/

SELECT subobject_name, statistic_name, value 
FROM v$segment_statistics 
WHERE object_name = 'T' 
AND owner = user 
AND statistic_name = 'logical reads';

PAUSE

REM
REM shows the number of distinct values stored in the table wri$_optstat_synopsis
REM

REM works up to 11.2.0.1

SELECT count(*)
FROM sys.wri$_optstat_synopsis$
WHERE synopsis# IN (SELECT sh.synopsis# 
                    FROM sys.wri$_optstat_synopsis_head$ sh
                    WHERE bo# IN (SELECT object_id
                                  FROM sys.user_objects
                                  WHERE object_name = 'T')
                    AND intcol# IN (SELECT column_id
                                    FROM sys.user_tab_columns
                                    WHERE TABLE_NAME = 'T' 
                                    AND column_name = 'NB')
                   )
GROUP BY synopsis#
ORDER BY synopsis#;

REM works as of 11.2.0.2

SELECT count(*)
FROM sys.wri$_optstat_synopsis$
WHERE bo# IN (SELECT object_id
              FROM sys.user_objects
              WHERE object_name = 'T')
AND intcol# IN (SELECT column_id
                FROM sys.user_tab_columns
                WHERE TABLE_NAME = 'T'
                AND column_name = 'NB')
GROUP BY group#
ORDER BY group#;

PAUSE

REM
REM show statistics in data dictionary, global statistics changed
REM

SELECT num_rows, sample_size, to_char(last_analyzed,'hh24:mi:ss') AS time 
FROM user_tables 
WHERE table_name = 'T';

SELECT partition_name, num_rows, sample_size, to_char(last_analyzed,'hh24:mi:ss') AS time 
FROM user_tab_partitions 
WHERE table_name = 'T'
ORDER BY partition_name;

PAUSE

SELECT column_name, num_distinct 
FROM user_tab_columns 
WHERE table_name = 'T';

SELECT  partition_name, column_name, num_distinct 
FROM user_part_col_statistics 
WHERE table_name = 'T' 
AND column_name = 'NB' 
ORDER BY partition_name;

PAUSE

REM
REM Cleanup
REM

DROP TABLE t PURGE;
